/******************************************************************************
 *  CVS version:
 *     $Id: error.c,v 1.2 2005/04/07 11:22:47 nickie Exp $
 ******************************************************************************
 *
 *  C code file : error.c
 *  Project     : Llama Compiler
 *  Version     : 1.0 alpha
 *  Description : Simple error handler
 *
 *  Comments: (in Greek iso-8859-7)
 *  ---------
 *    .
 *       .
 *      .
 *    
 */


/* ---------------------------------------------------------------------
   ---------------------------- Header files ---------------------------
   --------------------------------------------------------------------- */

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>

#include "general.h"
#include "error.h"


/* ---------------------------------------------------------------------
   ----------------------  , .. ----------------------
   --------------------------------------------------------------------- */

int  numErrors = 0;
int  maxErrors = 0;               /*     */
bool flagWarnings = true;
int  numWarnings = 0;
int  maxWarnings = 200;


/* ---------------------------------------------------------------------
   ---------       ----------
   --------------------------------------------------------------------- */

void internal_raw (const char * fn, int lc, const char * fmt, ...)
{
   va_list ap;

   va_start(ap, fmt);
   if (fmt[0] == '\r')
      fmt++;
   else
      fprintf(stderr, "%s:%d: ", filename, linecount);
   fprintf(stderr, "Internal error occurred at %s:%d, ", fn, lc);
   vfprintf(stderr, fmt, ap);
   fprintf(stderr, "\n");
   va_end(ap);
   numErrors++;
   terminate(1);
}

void fatal (const char * fmt, ...)
{
   va_list ap;

   va_start(ap, fmt);
   if (fmt[0] == '\r')
      fmt++;
   else
      fprintf(stderr, "%s:%d: ", filename, linecount);
   fprintf(stderr, "Fatal error, ");
   vfprintf(stderr, fmt, ap);
   fprintf(stderr, "\n");
   va_end(ap);
   numErrors++;
   terminate(1);
}

void error (const char * fmt, ...)
{
   va_list ap;

   va_start(ap, fmt);
   if (fmt[0] == '\r')
      fmt++;
   else
      fprintf(stderr, "%s:%d: ", filename, linecount);
   fprintf(stderr, "Error, ");
   vfprintf(stderr, fmt, ap);
   fprintf(stderr, "\n");
   va_end(ap);
   if (numErrors++ >= maxErrors) {
      message("\rToo many errors, aborting...");
      terminate(1);
   }
}

void warning (const char * fmt, ...)
{
   if (flagWarnings) {
      va_list ap;

      va_start(ap, fmt);
      if (fmt[0] == '\r')
         fmt++;
      else
         fprintf(stderr, "%s:%d: ", filename, linecount);
      fprintf(stderr, "Warning, ");
      vfprintf(stderr, fmt, ap);
      fprintf(stderr, "\n");
      va_end(ap);
      if (numWarnings++ >= maxWarnings) {
         message("\rToo many warnings, no more will be shown...");
         flagWarnings = false;
      }
   }
}

void message (const char * fmt, ...)
{
   va_list ap;

   va_start(ap, fmt);
   if (fmt[0] == '\r')
      fmt++;
   else
      fprintf(stderr, "%s:%d: ", filename, linecount);
   vfprintf(stderr, fmt, ap);
   fprintf(stderr, "\n");
   va_end(ap);
}
